@@ -1,14 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+# pylint: disable-all
+
 import os
 import re
 import sys
+import codecs
 import shutil
 import argparse
 from textwrap import dedent
 
 from chardet import detect
-from pysrt import SubRipFile, SubRipTime
+from pysrt import SubRipFile, SubRipTime, VERSION_STRING
+
+def underline(string):
+    return "\033[4m%s\033[0m" % string
 
 
 class SubRipShifter(object):
@@ -62,29 +68,34 @@ class SubRipShifter(object):
                 => creates movie.1.srt, movie.2.srt and movie.3.srt
     """)
     FRAME_RATE_HELP = "A frame rate in fps (commonly 23.9 or 25)"
-    
+    ENCODING_HELP = dedent("""\
+        Change file encoding. Useful for players accepting only latin1 subtitles.
+        List of supported encodings: http://docs.python.org/library/codecs.html#standard-encodings
+    """)
     def __init__(self):
         self.output_file_path = None
 
     def build_parser(self):
         parser = argparse.ArgumentParser(description=self.DESCRIPTION, formatter_class=argparse.RawTextHelpFormatter)
-        parser.add_argument('-i', action='store_true', dest='in_place',
+        parser.add_argument('-i', '--in-place', action='store_true', dest='in_place',
             help="Edit file in-place, saving a backup as file.bak (do not works for the split command)")
-
+        parser.add_argument('-e', '--output-encoding', metavar=underline('encoding'), action='store', dest='output_encoding',
+            type=self.parse_encoding, help=self.ENCODING_HELP)
+        parser.add_argument('-v', '--version', action='version', version='%%(prog)s %s' % VERSION_STRING)
         subparsers = parser.add_subparsers(title='commands')
 
         shift_parser = subparsers.add_parser('shift', help="Shift subtitles by specified time offset", epilog=self.SHIFT_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
-        shift_parser.add_argument('time_offset', action='store', metavar='offset',
+        shift_parser.add_argument('time_offset', action='store', metavar=underline('offset'),
             type=self.parse_time, help=self.TIMESTAMP_HELP)
         shift_parser.set_defaults(action=self.shift)
 
         rate_parser = subparsers.add_parser('rate', help="Convert subtitles from a frame rate to another", epilog=self.RATE_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
-        rate_parser.add_argument('initial', action='store', type=float, help=self.FRAME_RATE_HELP)
-        rate_parser.add_argument('final', action='store', type=float, help=self.FRAME_RATE_HELP)
+        rate_parser.add_argument(underline('initial'), action='store', type=float, help=self.FRAME_RATE_HELP)
+        rate_parser.add_argument(underline('final'), action='store', type=float, help=self.FRAME_RATE_HELP)
         rate_parser.set_defaults(action=self.rate)
 
         split_parser = subparsers.add_parser('split', help="Split a file in multiple parts", epilog=self.SPLIT_EPILOG, formatter_class=argparse.RawTextHelpFormatter)
-        split_parser.add_argument('limits', action='store', nargs='+', type=self.parse_time, help=self.LIMITS_HELP)
+        split_parser.add_argument(underline('limits'), action='store', nargs='+', type=self.parse_time, help=self.LIMITS_HELP)
         split_parser.set_defaults(action=self.split)
         
         parser.add_argument('file', action='store')
@@ -105,15 +116,23 @@ class SubRipShifter(object):
                         in self.RE_TIME_STRING.findall(time_string))
         return -ordinal if negative else ordinal
 
+    def parse_encoding(self, encoding_name):
+        print encoding_name
+        try:
+            codecs.lookup(encoding_name)
+        except LookupError, error:
+            raise argparse.ArgumentTypeError(error.message)
+        return encoding_name
+
     def shift(self):
         self.input_file.shift(milliseconds=self.arguments.time_offset)
-        self.input_file.write_into(self.output_file, encoding=self.output_encoding)
+        self.input_file.write_into(self.output_file)
         self.output_file.close()
 
     def rate(self):
         ratio = self.arguments.final / self.arguments.initial
         self.input_file.shift(ratio=ratio)
-        self.input_file.write_into(self.output_file, encoding=self.output_encoding)
+        self.input_file.write_into(self.output_file)
         self.output_file.close()
 
     def split(self):
@@ -124,7 +143,7 @@ class SubRipShifter(object):
             part_file = self.input_file.slice(ends_after=start, starts_before=end)
             part_file.shift(milliseconds=-start)
             part_file.clean_indexes()
-            part_file.save(path=file_name)
+            part_file.save(path=file_name, encoding=self.output_encoding)
 
     def create_backup(self):
         backup_file = self.arguments.file + self.BACKUP_EXTENSION
@@ -135,21 +154,21 @@ class SubRipShifter(object):
 
     @property
     def output_encoding(self):
-        # TODO: add an option to change encoding
-        return self.input_file.encoding
+        return self.arguments.output_encoding or self.input_file.encoding
 
     @property
     def input_file(self):
         if not hasattr(self, '_source_file'):
             encoding = detect(open(self.arguments.file).read()).get('encoding')
-            self._source_file = SubRipFile.open(self.arguments.file, encoding=encoding)
+            self._source_file = SubRipFile.open(self.arguments.file,
+                encoding=encoding, error_handling=SubRipFile.ERROR_LOG)
         return self._source_file
 
     @property
     def output_file(self):
         if not hasattr(self, '_output_file'):
             if self.output_file_path:
-                self._output_file = open(self.output_file_path, 'w+')
+                self._output_file = codecs.open(self.output_file_path, 'w+', encoding=self.output_encoding)
             else:
                 self._output_file = sys.stdout
         return self._output_file
